#include "AWSprite.h"
#include "AWFont.cpp"
#include "stdlib.h"
#include "SDL_image.h"
#include "joystickmapping.h"

#define playerBatSPEED 		(1)
#define computerBatSPEED 	(1)
#define GOALMOUTH			(40)

int main ( int argc, char *argv[] )
{
  	
  Uint32 WIDTH  = 320;
  Uint32 HEIGHT = 240;

  int playerScore = 0;
  int computerScore = 0;
  
  int playerBatMoveY = 0;
  int playerBatY = 110;
  int computerBatY = 110;
     
  float ballSpeedX = 4.3f;
  float ballSpeedY = 2.8f;
  
  int easiness;
  
  // see if there's a parameter
  // if there is, it defines how easy the game should be: 8=very hard, 1=very easy
  if(argc > 1) {
  	easiness = atoi(argv[1]);
	if(easiness < 0) easiness = 1;
	if(easiness > 8) easiness = 8;
  } else {
  	easiness = 5;
  }

  /* initialize SDL */
  SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK);
  SDL_JoystickOpen(0);
  SDL_ShowCursor(SDL_DISABLE);
  
  /* set the title bar */
  SDL_WM_SetCaption("Pong", "Pong");
  /* create window */
  SDL_Surface* screen = SDL_SetVideoMode(WIDTH, HEIGHT, 0, SDL_SWSURFACE);

 
  /* This is the background surface */
  SDL_Surface* temp = SDL_LoadBMP("board.bmp");
  /* convert bitmap to display format */
  SDL_Surface* bg = SDL_DisplayFormat(temp);
  /* free the temp surface */
  SDL_FreeSurface(temp);

  // Load in the font
  AWFont *font = new AWFont("vera.bmp", "vera.dat");


  // This is the ball sprite
  AWSprite *ball = new AWSprite("ball.bmp", 1);
  ball->set_transparent_colour(0, 0, 0);
  ball->set_velocities(ballSpeedX, ballSpeedY);
  ball->set_auto_move(20);
  ball->set_world_position(5, 5);
  
  AWSprite *playerBat = new AWSprite("bat.bmp", 1);
  playerBat->set_transparent_colour(0, 0, 0);
  playerBat->set_world_position(WIDTH-GOALMOUTH, playerBatY);
  
  AWSprite *computerBat = new AWSprite("bat.bmp", 1);
  computerBat->set_transparent_colour(0, 0, 0);
  computerBat->set_world_position(GOALMOUTH, computerBatY);
  int halfBatHeight = computerBat->get_height()/2;

  SDL_Event event;
  int gameover = 0;

  // game loop */
  while (!gameover)
  {
    if (SDL_PollEvent(&event)) {
      switch (event.type) {
        case SDL_JOYBUTTONDOWN:
            switch(event.jbutton.button)
            {
                 case GP2X_BUTTON_START:
					 gameover = 1;
					 break;
				 case GP2X_BUTTON_DOWN:
				 case GP2X_BUTTON_R:
				 case GP2X_BUTTON_X:
					playerBatMoveY = +playerBatSPEED;
			  		break;
             	 case GP2X_BUTTON_UP:
				 case GP2X_BUTTON_L:
				 case GP2X_BUTTON_Y:
					playerBatMoveY = -playerBatSPEED;
					break;
            }
            break;
        case SDL_JOYBUTTONUP:
            switch(event.jbutton.button)
            {
				 case GP2X_BUTTON_DOWN:
				 case GP2X_BUTTON_R:
				 case GP2X_BUTTON_X:
					if(playerBatMoveY > 0) playerBatMoveY = 0;
			  		break;
             	 case GP2X_BUTTON_UP:
				 case GP2X_BUTTON_L:
				 case GP2X_BUTTON_Y:
					if(playerBatMoveY < 0) playerBatMoveY = 0;
					break;
            }
            break;
      }
    }

	// Move the player's bat
	playerBatY = playerBat->get_y() + playerBatMoveY;
	if(playerBatY < 1) playerBatY = 1;
	if(playerBatY > HEIGHT - playerBat->get_height() - 1) playerBatY = (HEIGHT - playerBat->get_height() - 1);
	playerBat->set_world_position(playerBat->get_x(), playerBatY);
	

	// Move the computer's bat
	if(ball->velX < 0 && ball->worldX < (WIDTH*easiness) / 8) {
		if((computerBat->worldY + halfBatHeight) > ball->worldY) {
			computerBatY = computerBat->get_y() - computerBatSPEED;
			if(computerBatY < 0) computerBatY = 0;
			computerBat->set_world_position(computerBat->get_x(), computerBatY);
		} else if((computerBat->worldY + halfBatHeight) < ball->worldY) {
			computerBatY = computerBat->get_y() + computerBatSPEED;
			if(computerBatY > HEIGHT - computerBat->get_height() - 1) computerBatY = (HEIGHT - computerBat->get_height() - 1);
			computerBat->set_world_position(computerBat->get_x(), computerBatY);
		}
	}	
	
	
    // draw the background
    SDL_BlitSurface(bg, NULL, screen, NULL);

	// Sort out bouncing off the top and bottom...
	if(ball->worldY <= 1) {
		ball->worldY = 2;
		ball->velY = -ball->velY;
	}
	if(ball->worldY+ball->get_height() >= HEIGHT-1) {
		ball->worldY = HEIGHT-ball->get_height() - 2;
		ball->velY = -ball->velY;
	}

	// Sort out scoring
	if(ball->worldX < 2) {
		playerScore++;
		ball->set_auto_move(0);
		SDL_Delay(2000);
		ball->set_world_position( 5, rand() % HEIGHT);
		ball->make_visible();
		ball->set_velocities(ballSpeedX, ballSpeedY);
		ball->set_auto_move(20);
	}
	if(ball->worldX > WIDTH-5) {
		computerScore++;
		ball->set_auto_move(0);
		SDL_Delay(2000);
		ball->set_world_position(WIDTH-5, rand() % HEIGHT);
		ball->make_visible();
		ball->set_velocities(-ballSpeedX, ballSpeedY);
		ball->set_auto_move(20);
	}		
		

	// Check for collision with the player's bat
	if(ball->velX > 0 && playerBat->pp_collision(ball)) {
		ball->velX = -ball->velX;
	}

	// Check for collision with the computer's bat
	if(ball->velX < 0 && computerBat->pp_collision(ball)) {
		ball->velX = -ball->velX;
	}
	

	font->print_integer(computerScore, WIDTH/4, 5);
	font->print_integer(playerScore, (WIDTH*3)/4, 5);

	// Move and display the sprites
	ball->update_everything();
	playerBat->update_everything();
	computerBat->update_everything();
	
    /* update the screen */
    SDL_Flip(screen);
	SDL_Delay(5);
	
  }

  /* free the background surface */
  SDL_FreeSurface(bg);

  /* cleanup SDL */
  SDL_Quit();

  return 0;
}

